/* Parable of the Broken Window */
var debug = (location.search.indexOf('debug') > 0) ? true : false;
(function(){
    //global
    var canvas,
        context,
        timelast,
        dt,
        width,
        height,

        once,
        lastscene,
        countTeleports;

    //constants
    var ROOM_WIDTH,
        ROOM_HEIGHT,
        PADDING_LEFT,
        PADDING_TOP;

    //settings
    var mute,
        score;

    //runtime vars
    var roomsY,
        roomsX;

    //objects
    var controls,
        player,
        rooms,
        bgmusicCurrent,
        bgmusic,
        bgmusic2,
        messages;

    //images
    var imgBG,
        imgChar,
        imgFinalDoor,
        imgChestOpen,
        imgSword,
        imgShield,
        imgBoots,
        imgHelm,
        imgArmor,
        //imgBullet, //global
        imgRoomItems, //array of images 
        imgCharPack; //array of images 
    
    window.onload = onload;

    function onload() { 
        //setup objects once
        canvas = document.getElementById('game');
        context = canvas.getContext('2d');
        onresize();

        //constants
        ROOM_WIDTH = 224;
        ROOM_HEIGHT = 128;
        PADDING_LEFT = PADDING_TOP = 20;

        mute = false;
        score = {
            key: {
                el: document.getElementById('key'),
                val: 0
            }
        }
        countTeleports = 0;

        //images
        loadImage('game/sprites/w-block-dark-01.png', function(img){
            imgBG = context.createPattern(img, 'repeat');
        });
        imgChar = loadImage('game/sprites/c-sprite.png');
        imgFinalDoor = loadImage('game/sprites/the-door.png');

        imgChestOpen = loadImage('game/sprites/i-chest-open.png');
        imgSword = loadImage('game/sprites/g-sword.png');
        imgShield = loadImage('game/sprites/g-shield.png');
        imgBoots = loadImage('game/sprites/g-boots.png');
        imgHelm = loadImage('game/sprites/g-helm.png');
        imgArmor = loadImage('game/sprites/g-armor.png');
        imgBullet = loadImage('game/sprites/m-fireball.png');

        imgRoomItems = [
            loadImage('game/sprites/w-block-01.png'), 
            loadImage('game/sprites/i-door.png'),
            loadImage('game/sprites/i-key.png'),
            loadImage('game/sprites/i-chest.png'),
            imgSword,
            imgShield,
            loadImage('game/sprites/g-boots-pickup.png'),
            loadImage('game/sprites/g-helm-pickup.png'),
            imgArmor,
            loadImage('game/sprites/i-spikes.png'),
            imgFinalDoor,
            ,
            loadImage('game/sprites/m-ghoul.png')
        ];
        imgCharPack = [
            imgSword,
            imgShield,
            imgHelm,
            imgArmor,
            imgBoots
        ];

        /***** SETUP OBJECTS *****/
        controls = {
            left: false,
            leftPressed: false,
            right: false,
            rightPressed: false,
            jump: false,
            jumpPressed: false,
            crouch: false,
            crouchPressed: false,
            attack: false,
            attackPressed: false,
            use: false,
            usePressed: false
        };
        player = new Player({
            context: context,
            image: imgChar,
            x: 32,
            y: height/2,
            width: 16,
            height: 32,
            imgGear: imgCharPack
        });
        rooms = [[]];
        bgmusic = document.getElementById('bg-music');
        bgmusic2 = document.getElementById('bg-music');
        bgmusicCurrent = bgmusic;
        bgmusicCurrent.play();

        messages = [];

        // imgFlower = new Sprite({
        //     context: context,
        //     image: loadImage('game/sprites/deco-flower-sprite.png'),
        //     fps: 12,
        //     width: 46,
        //     height: 69,
        //     x: 100,
        //     y: canvas.height - 32
        // });

        /***** SETUP INTERFACE *****/
        window.onresize = onresize;
        document.getElementById('btn-mute').addEventListener('click', function(e){
            e.preventDefault();
            if (this.getAttribute('class') == "muted") {
                this.setAttribute('class', "");
                this.innerHTML = "Mute";
                mute = false;

                bgmusicCurrent.currentTime = 0;
                bgmusicCurrent.play();

            } else {
                this.setAttribute('class', "muted");
                this.innerHTML = "Unmute";
                mute = true;

                bgmusicCurrent.pause();
            }
        });
        document.getElementById('btn-help').addEventListener('click', function(e){
            e.preventDefault();
            alert('Parable of the Broken Window\n\nEscape the dungeon by finding the big red door.\nThe big door requires 2 keys, and appears after the 5th column of rooms.\n\n- Arrows or WASD to move\n- Resize the browser window to reveal more rooms\n- Space to attack (Only with Sword)\n- Down/S to block (Only with Shield)\n- Armor will allow you to take more hits from monsters\n- Collect keys to unlock chests and doors\n- Walking to the edge of a screen with no wall, will teleport you to the other side');
        });


        /***** CONTROLS *****/
        document.body.addEventListener('touchmove',function(e){e.preventDefault();}); //prevent touch app from dragging game screen
        window.addEventListener('keydown', function(e){
            var kc = e.keyCode;
            if (debug) { console.log(kc); }

            // if (kc != 9 && kc != 91 && kc != 82 && kc != 13) { //not tab cmd or R or Enter
            //     e.preventDefault();    
            // }
            
            if (kc == 37 || kc == 65) { //left A
                controls.left = true;
            } else if (kc == 39 || kc == 68) { //right D
                controls.right = true;
            }

            if (kc == 38 || kc == 87) { //up W
                if (!controls.jumpPressed) {
                    controls.jump = true;
                    controls.jumpPressed = true;
                }
            } else if (kc == 40 || kc == 83) { //down S
                // if (!controls.crouchPressed) {
                    controls.crouch = true;
                    //controls.crouchPressed = true;
                //}
            }

            if (kc == 32) { //space
                if (!controls.attackPressed) {
                    controls.attackPressed = true;
                    controls.attack = true;
                }
            }
        });
        window.addEventListener('keyup', function(e){
            var kc = e.keyCode;
            
            if (kc == 37 || kc == 65) { //left A
                controls.left = false;
                controls.leftPressed = false;
            } else if (kc == 39 || kc == 68) { //right D
                controls.right = false;
                controls.rightPressed = false;
            }

            if (kc == 38 || kc == 87) { //up W
                controls.jump = false;
                controls.jumpPressed = false;
            } else if (kc == 40 || kc == 83) { //down S
                controls.crouch = false;
                //controls.crouchPressed = false;
            }

            if (kc == 32) { //space
                controls.attackPressed = false;
                controls.attack = false;
            }
        });

        /***** START NEW ROUND *****/
        //startGame();

        /***** START LOOP *****/
        timelast = new Date().getTime();
        (function animloop() {
            requestAnimFrame(animloop);

            var ct = new Date().getTime();
            dt = (ct - timelast);
            timelast = ct;

            draw(dt);
        })();
    }


    /* Draw Loop */
    function draw(dt) {
        context.save();

        if (totalAssets.loaded < totalAssets.loading) { //loading;
            context.fillStyle = '#000000';
            context.fillRect(0,0,width,height);
            context.fillStyle = '#ffffff';
            context.fillRect(0,0,totalAssets.loaded/totalAssets.loading * width, 20);

            if (debug) {console.log(totalAssets.loaded + '/' + totalAssets.loading);}
        } else {

            if (!once) {
                once = true;

                startGame();
            }

            if (context.imageSmoothingEnabled) {
                context.imageSmoothingEnabled = false;
            } else if (context.webkitImageSmoothingEnabled) {
                context.webkitImageSmoothingEnabled = false;
            } else if (context.mozImageSmoothingEnabled) {
                context.mozImageSmoothingEnabled = false;
            }

            //clear frame
            context.fillStyle = imgBG;//"#000000";
            context.fillRect(PADDING_LEFT-4, PADDING_TOP-6, width, height);

            //center frame
            // context.translate(
            //     width/2 - ROOM_WIDTH/2,
            //     height/2 - ROOM_HEIGHT/2
            // );
            context.translate(PADDING_LEFT, PADDING_TOP);

            if (!lastscene) {
                //UPDATE

                if (bgmusicCurrent.currentTime > 53) {
                    if (bgmusicCurrent === bgmusic) {
                        bgmusicCurrent = bgmusic2;
                    } else {
                        bgmusicCurrent = bgmusic;
                    }
                    bgmusicCurrent.currentTime = 0;
                    bgmusicCurrent.play();
                }

                //player
                if (!player.dead) {
                    if (controls.crouch) {
                        if (player.shield) {
                            player.blocking = true;
                        }

                        player.vx += (0-player.vx) * 0.5;
                    } else {
                        player.blocking = false;


                        if (controls.right) {
                            player.vx += (0.3 - player.vx) * 0.5;
                        } else if (controls.left) {
                            player.vx += (-0.3 - player.vx) * 0.5;
                        } else {
                            player.vx += (0-player.vx) * 0.5;
                        }

                        if (player.grounded && controls.jump) {
                            controls.jump = false;
                            player.jump();
                        }

                        if (controls.attack) {
                            controls.attack = false;
                            player.attack();
                        }
                    }
                }

                player.update(dt);

                //brick collisions
                for (var y=0; y<rooms.length; y++) {
                    for (var x=0; x<rooms[y].length; x++) {
                        for (var j=rooms[y][x].blocks.length-1; j>=0; j--) {
                            var block = rooms[y][x].blocks[j];
                            block.update(dt);

                            var collision = checkCollision(player, block, rooms[y][x].x, rooms[y][x].y);

                            if (collision.hit) {
                                if (block.type === 0 || block.type === 1) {

                                    if (block.type === 1 && score.key.val > 0) {
                                        score.key.val--;
                                        rooms[y][x].blocks.splice(j,1);
                                        score.key.el.innerHTML = score.key.val;
                                    } else {
                                        if (collision.direction === 0) {
                                            player.y = collision.y;
                                            player.vy = 0;
                                            player.grounded = true;
                                        } else if (collision.direction === 2) {
                                            player.y = collision.y;
                                            player.vy *= -0.5;
                                        } else if (collision.direction === 1 || collision.direction === 3) {
                                            player.x = collision.x;
                                            player.vx *= -0.5;
                                        }
                                    }
                                } else if (block.type === 2) { //key
                                    score.key.val++;
                                    rooms[y][x].blocks.splice(j,1);
                                    score.key.el.innerHTML = score.key.val;
                                } else if (block.type === 3 && score.key.val > 0 && !block.used) { //chest
                                    score.key.val--;
                                    block.img = imgChestOpen;
                                    block.used = true;
                                    score.key.el.innerHTML = score.key.val;
                                    
                                    for (var i=0; i<Math.floor(Math.random()*2)+1; i++) {
                                        var types = [2, 6, 7, 8]; //available items in chest
                                        if (!player.shield) {
                                            types.push(5); //add shields if player doesn't have it yet
                                        }
                                        var type = types[Math.floor(Math.random()*types.length)];
                                        var newitem = new Item({
                                            context: context,
                                            type: type,
                                            img: imgRoomItems[type],
                                            x: block.x + i*16 - 16, 
                                            y: block.y - 32
                                        });
                                        rooms[y][x].blocks.push(newitem);
                                    }
                                } else if (!player.sword && block.type === 4) { //sword
                                    player.sword = true;
                                    rooms[y][x].blocks.splice(j,1);
                                } else if (!player.shield && block.type === 5) { //shield
                                    player.shield = true;
                                    rooms[y][x].blocks.splice(j,1);
                                } else if (!player.boots && block.type === 6) { //boots
                                    player.boots = true;
                                    rooms[y][x].blocks.splice(j,1);
                                } else if (!player.helm && block.type === 7) { //helm
                                    player.helm = true;
                                    rooms[y][x].blocks.splice(j,1);
                                } else if (!player.armor && block.type === 8) { //armor
                                    player.armor = true;
                                    rooms[y][x].blocks.splice(j,1);
                                } else if (block.type === 9) { //spikes
                                    player.vy = -0.5;
                                    if (player.boots) {
                                        player.boots = false;
                                    } else {
                                        player.dead = true;
                                        setTimeout(function(){
                                            startGame();
                                        }, 3000);
                                    }
                                } else if ((block.type === 11) || (block.type === 12)) { //enemies
                                    console.log('block type intersect');
                                    if (player.attacking) {
                                        rooms[y][x].blocks.splice(j,1);
                                    } else {
                                        takeDamage();
                                    }
                                } else if (!player.dead && block.type === 10 && score.key.val >= 2) { //final door
                                    score.key.val-=2;
                                    score.key.el.innerHTML = score.key.val;
                                    lastscene = true;
                                    endGame();
                                }

                            }

                            //something else to do in the loop
                            if ((block.type === 12) || (block.type === 11)) { //monsters
                                if (player.x > rooms[y][x].x + block.x) {
                                    block.direction = -1;
                                } else {
                                    block.direction = 1;
                                }

                                if (block.type === 12) {
                                    for (var i=block.bullets.length-1; i>=0; i--) {
                                        var bullethit = checkCollision(player, block.bullets[i], rooms[y][x].x + block.x, rooms[y][x].y + block.y);
                                        if (bullethit.hit) {
                                            console.log(bullethit)
                                            block.bullets.splice(i,1);
                                            takeDamage();
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                //phase walls
                if (player.x > (roomsX * ROOM_WIDTH)) {
                    player.x = 0;

                    if (countTeleports < 10) { countTeleports++; }
                } else if (player.x < 0) {
                    player.x = (roomsX * ROOM_WIDTH);

                    if (countTeleports < 10) { countTeleports++; }
                }
                if (player.y > (roomsY * ROOM_HEIGHT)) {
                    player.y = 0;

                    if (countTeleports < 10) { countTeleports++; }
                } else if (player.y < 0) {
                    player.y = (roomsY * ROOM_HEIGHT);

                    if (countTeleports < 10) { countTeleports++; }
                }

                if (countTeleports > 1 && countTeleports < 10) {
                    countTeleports = 11;
                    messageFlash('TRY RESIZING', width/2 - PADDING_LEFT, height/2-20, 8000);
                    messageFlash('THE WINDOW ;D', width/2 - PADDING_LEFT, height/2, 8000);
                }


                //RENDER
                
                //rooms
                for (var y=0; y<rooms.length; y++) {
                    for (var x=0; x<rooms[y].length; x++) {
                        rooms[y][x].render();
                    }
                }

                //player
                player.render();

                //overlays
                for (var i=messages.length-1; i>=0; i--) {
                    if (messages[i].deletable) {
                        messages.splice(i,1);
                    } else {
                        messages[i].render();
                    }
                }



                //black bars
                context.fillStyle = "#000000";
                context.fillRect(roomsX * ROOM_WIDTH, 0, ROOM_WIDTH, height);
                context.fillRect(0, -PADDING_TOP, width, PADDING_TOP);
                context.fillRect(0, roomsY * ROOM_HEIGHT, width, ROOM_HEIGHT);
                context.fillRect(-PADDING_LEFT, 0, PADDING_LEFT, height);
            
            } else { //if lastscene
                context.font = '18px PressStart2P';
                context.fillStyle = '#fff';
                context.textAlign = 'center';

                context.fillText('You exit to find yourself in an old man\'s house.', width/2, height/2-200);
                context.fillText('"Who are you?" shouts the old man,', width/2, height/2-140);
                context.fillText('"What are you doing in my basement?..."', width/2, height/2-100);
                context.fillText('"And with my stuff?!"', width/2, height/2-60);
                context.fillText('"Thief! Get back here!"', width/2, height/2-20);

                context.font = '30px PressStart2P';
                context.fillText('The End', width/2, height/2+100);
            }
        }

        context.restore();
    }
        


    /***** COMMMON FUNCTIONS *****/
    function startGame(){

        //set variables
        width = 224 + PADDING_LEFT*2;
        height = 128 + PADDING_TOP*2;
        rooms = [[]];
        roomsX = 0;
        roomsY = 0;

        player.dead = false;
        player.x = 64;
        player.y = 96;
        player.vx = 0;
        player.vy = 0;
        

        //setup stage
        window.resizeTo(width, height + 52);

        rooms = [[newRoom(0, 0, 1)]];

    }

    function endGame() {

    }

    function takeDamage() {
        if (!player.blocking) {
            if (player.helm) {
                player.helm = false;
            } else if (player.armor) {
                player.armor = false;
            } else {
                player.dead = true;
                setTimeout(function(){
                    startGame();
                }, 3000);
            }
            player.vy = -0.5;
        }
    }

    function newRoom(x, y, type) {
        var rm;

        if (type === 1) {
            roomsX = 1;
            roomsY = 1;
            rm = new Room({
                context: context,
                images: imgRoomItems,
                //bg: imgBG,
                blocks: [
                    [0,0,0,0,0,0,0,0,0,0,0,0,0,0],
                    [0,0,0,0,0,0,0,0,0,0,0,0,0,0],
                    [0,0,0,0,0,0,0,0,0,0,0,0,0,0],
                    [0,0,0,0,0,0,0,0,0,0,0,0,0,0],
                    [0,0,0,0,0,0,0,0,0,0,0,0,0,0],
                    [0,0,0,0,0,0,0,0,0,0,0,0,0,0],
                    [0,0,0,0,0,0,0,0,5,0,0,0,0,0],
                    [1,1,1,1,1,1,1,1,1,1,1,1,1,1]
                ],
                width: ROOM_WIDTH,
                height: ROOM_HEIGHT,
                x: x,
                y: y
            });
        } else if (type === 2) {
            rm = new Room({
                context: context,
                images: imgRoomItems,
                //bg: imgBG,
                blocks: [
                    [0,0,0,0,0,0,0,0,0,0,0,0,0,0],
                    [0,0,0,0,0,0,0,0,0,0,0,0,0,0],
                    [0,0,0,0,0,0,0,0,0,0,0,0,0,0],
                    [0,0,0,0,0,0,0,0,0,0,0,0,0,0],
                    [0,0,0,0,0,0,0,0,0,0,0,0,0,0],
                    [0,0,0,0,0,0,0,0,0,0,0,0,0,0],
                    [0,0,0,0,0,0,12,11,12,0,0,0,0,0],
                    [1,1,1,1,1,1,1,1,1,1,1,1,1,1]
                ],
                width: ROOM_WIDTH,
                height: ROOM_HEIGHT,
                x: x,
                y: y
            });
        } else {
            var hasFloor = Math.random();
            var startGap = 0;
            var endGap = 0;

            var hasWall = 0;
            var door1 = 0;
            var door2 = 0;
            var item = [0,0,0,0];

            var monster = 0;

            var floor = [0,0,0,0,0,0,0,0,0,0,0,0,0,hasWall];
            if (hasFloor < 0.25) {
                floor = [1,1,1,1,1,1,1,1,1,1,1,1,1,1];
            } else {
                startGap = 2 + Math.floor(Math.random()*8);
                endGap = Math.floor(startGap+2 + (Math.random()*(9-2-startGap)) );

                floor = [];
                for (var i=0; i<14; i++) { //only do 13 blocks, last one is determined by the wall
                    if (hasWall && (i === 13)) {
                        floor.push(1);
                    } else if ((i < startGap) || (i > endGap)) {
                        floor.push(1);  
                    } else {
                        floor.push(0);
                    }
                }
            }

            if (Math.random() < 0.75) {
                hasWall = 1;

                if (Math.random() < 0.5) { //50% chance to be a locked door instead of wall
                    door1 = 0;
                    door2 = 2;
                } else {
                    door1 = door2 = hasWall;
                }
            }

            var randitem = Math.random();
            var key = 0.2;
            var chest = 0.3;
            var shield = (player.shield) ? 0 : 0.05;
            var boots = 0.02;
            var gear = 0.1;
            var spikes = 0.2;
            
            if (randitem < key) {
                item[Math.floor(Math.random()*4)] = 3;
            } else if (randitem < key + chest) {
                item[Math.floor(Math.random()*4)] = 4;

                if (Math.random() < 0.8) { //75% monster
                    monster = 13;
                }
            } else if (randitem < key + chest + shield) {
                item[Math.floor(Math.random()*4)] = 6;

                if (Math.random() < 0.8) { //75% monster
                    monster = 13;
                }
            } else if (randitem < key + chest + shield + boots) {
                item[Math.floor(Math.random()*4)] = 7;
                if (Math.random() < 0.8) { //75% monster
                    monster = 13;
                }
            } else if (randitem < key + chest + shield + boots + gear) {
                item[Math.floor(Math.random()*4)] = 8 + (Math.floor(Math.random()*2)); //pick 1 of the next 2 items
            } else if (randitem < key + chest + shield + boots + gear + spikes) {
                item[Math.floor(Math.random()*4)] = 10;
            }

            if (startGap && (item[1] || item[2])) {//if there is a gap, and an item in slot 1 or 2, then move it to 0 or 3
                if (item[1]) {
                    item[0] = item[1];
                    item[1] = 0;
                }
                if (item[2]) {
                    item[3] = item[2];
                    item[2] = 0;
                }
            }


            var randBlock = function() {
                return Math.floor(Math.random() * 2);
            }
            var leftArc = randBlock();
            var rightArc = randBlock();

            var platform = randBlock();

            rm = new Room({
                context: context,
                images: imgRoomItems,
                //bg: imgBG,
                blocks: [
                    [leftArc,leftArc,leftArc,0,0,0,0,0,0,0,0,rightArc,rightArc,hasWall],
                    [leftArc,leftArc,0,0,0,0,0,0,0,0,0,0,rightArc,hasWall],
                    [randBlock(),0,0,0,0,randBlock(),0,0,0,0,0,0,0,hasWall],
                    [0,0,0,0,randBlock(),platform,platform,0,0,0,0,0,0,hasWall],
                    [0,0,0,randBlock(),0,0,0,0,0,0,0,0,0,hasWall],
                    [0,0,0,0,0,0,0,0,0,0,0,0,0,door1],
                    [0,item[0],0,item[1],0,0,0,0,monster,item[2],0,item[3],0,door2],
                    floor
                ],
                width: ROOM_WIDTH,
                height: ROOM_HEIGHT,
                x: x,
                y: y
            });
        }

        //rooms.push(rm);
        return rm;
    }


    function checkIntersect(soft, hard) {
        var hit = false;
        if (
            (soft.x > (hard.x - hard.width/2 - soft.width/2)) &&
            (soft.x < (hard.x + hard.width/2 + soft.width/2)) &&
            (soft.y > (hard.y - hard.height/2 - soft.height/2)) &&
            (soft.y < (hard.y + hard.height/2 + soft.height/2)) 
        ) {
            hit = true;
        }
        return hit;
    };

    function checkCollision(soft, hard, dispx, dispy) {
        var dir = false;
        var hit = false;
        var x = 0;
        var y = 0;

        var softWidth = soft.width; //write this to variable so it can be changed for hacky attacking size
        //start hacky width for attacking
        if (soft.attacking && (hard.type === 11 || hard.type === 12)) {
            softWidth = soft.attackingWidth;
        }

        var push = 0; //amount to push when collided

        dispx = dispx || 0;
        dispy = dispy || 0;

        var d0 = (hard.y + dispy + hard.top - soft.height/2);
        var d1 = (hard.x + dispx + hard.left - softWidth/2);
        var d2 = (hard.y + dispy + hard.top + hard.height + soft.height/2);
        var d3 = (hard.x + dispx + hard.left + hard.width + softWidth/2);

        if (
            (soft.y > d0) &&
            (soft.x > d1) &&
            (soft.y < d2) &&
            (soft.x < d3)
        ) {

            var dist = [
                soft.y - d0,
                soft.x - d1,
                -soft.y + d2,
                -soft.x + d3
            ];

            var smallest = dist[0];
            for (var i=1; i<dist.length; i++) {
                if (dist[i] && (dist[i] < smallest)) {
                    smallest = dist[i];
                }
            }

            if (smallest) { //if it isn't zero
                hit = true;

                if (smallest === dist[0]) {
                    dir = 0;
                    y = d0;
                } else if (smallest === dist[1]) {
                    dir = 1;
                    x = d1 - push;
                } else if (smallest === dist[2]) {
                    dir = 2;
                    y = d2;
                } else if (smallest === dist[3]) {
                    dir = 3;
                    x = d3 + push;
                }
            }

            // if (Math.abs(soft.vx) > Math.abs(soft.vy)) {
            //     if (soft.vx > 0) {
            //         dir = 1;
            //         x = hard.x - hard.width/2 - softWidth/2;
            //     } else {
            //         dir = 3;
            //         x = hard.x + hard.width/2 + softWidth/2;
            //     }
            // } else {
            //     if (soft.vy > 0) {
            //         dir = 0;
            //         y = hard.y - hard.height/2 - soft.height/2;
            //     } else {
            //         dir = 2;
            //         y = hard.y + hard.height/2 + soft.height/2;
            //     }
            // }
        }


        // if (softx > (hardx - hardw/2 - softw/2)) {
        //     x = hardx-hardw/2-softw/2;
        // } else if (softx < (hardx + hardw/2 + softw/2)) {
        //     x = hardx + hardw/2 + softw/2;
        // } else if (softy > (hardy - hardh/2 - softh/2)) {
        //     y = hardy - hardh/2 - softh/2;
        // } else if (softy < (hardy + hardh/2 + softh/2)) {
        //     y = hardy + hardh/2 + softh/2;
        // }
        

        // if (
        //     ((soft.y + soft.height/2) > (hard.y - hard.height/2)) &&
        //     ((soft.y - soft.height/2) < (hard.y - hard.height/2)) &&
        //     (soft.vy > 0)
        // ) {
        //     y = hard.y - hard.height/2 - soft.height/2;
        //     hit = true;
        //     dir = 0;
        // } else if (
        //     ((soft.y - soft.height/2) < (hard.y + hard.height/2)) &&
        //     ((soft.y + soft.height/2) > (hard.y + hard.height/2)) &&
        //     (soft.vy < 0)
        // ) {
        //     y = hard.y + hard.height/2 + soft.height/2;
        //     hit = true;
        //     dir = 2;
        // } else if (
        //     ((soft.x + softWidth/2) > (hard.x - hard.width/2)) &&
        //     ((soft.x - softWidth/2) < (hard.x - hard.width/2)) &&
        //     (soft.vx > 0)
        // ) {
        //     x = hard.x - hard.width/2 - softWidth/2;
        //     hit = true;
        //     dir = 3;
        // } else if (
        //     ((soft.x - softWidth/2) < (hard.x + hard.width/2)) &&
        //     ((soft.x + softWidth/2) > (hard.x + hard.width/2)) &&
        //     (soft.vx < 0)
        // ) {
        //     x = hard.x + hard.width/2 + softWidth/2;
        //     hit = true;
        //     dir = 1;
        // }

        return {
            hit: hit,
            direction: dir,
            x: x,
            y: y
        };
    }


    function messageFlash(s, x, y, time, col) {
        var color = "#fff";
        if (col) {
            color = col;
        }

        time = time || 1000;

        var msg = {
            x: x || 0,
            y: y,
            text: s,
            color: color,
            opacity: 1,
            deletable: false,
            timer: 0,
            totalTime: time,
            render: function () {
                if (!this.deletable) {
                    this.timer += dt;
                    if (this.timer < 50) {
                        this.opacity = 0;
                    } else if (this.timer < 100) {
                        this.opacity = 1;
                    } else if (this.timer < 150) {
                        this.opacity = 0;
                    } else if (this.timer < this.totalTime-250) {
                        this.opacity = 1;
                    } else if (this.timer < this.totalTime-200) {
                        this.opacity = 0;
                    } else if (this.timer < this.totalTime-150) {
                        this.opacity = 1;
                    } else if (this.timer < this.totalTime-100) {
                        this.opacity = 0;
                    } else if (this.timer < this.totalTime-50) {
                        this.opacity = 1;
                    } else if (this.timer < this.totalTime) {
                        this.opacity = 0;
                    } else {
                        this.opacity = 1;
                        this.deletable = true;
                    }

                    context.save();
                        context.font = '14px PressStart2P';
                        context.textAlign = 'center';
                        context.globalAlpha = this.opacity;
                        context.fillStyle = this.color;
                        context.fillText(this.text, this.x, this.y);
                    context.restore();
                }
            }
        };
        messages.push(msg);
    }

    function onresize() {
        if (canvas) {
            width = window.innerWidth;
            height = window.innerHeight;
            canvas.width = width;
            canvas.height = height;
            if ((width > ROOM_WIDTH + PADDING_LEFT) && (height > ROOM_HEIGHT + PADDING_TOP)) {

                if (width >= (roomsX * ROOM_WIDTH + ROOM_WIDTH + PADDING_LEFT)) {
                    var shouldFinal = false;

                    if (roomsX > 4) {
                        if (Math.random() < 0.75) { //75% chance of spawning final door
                            shouldFinal = true;
                        }
                    }

                    for (var i=0; i<roomsY; i++) {
                        var rm;
                        if (i>1 && shouldFinal && (Math.random() < 0.5)) {
                            rm = newRoom(roomsX * ROOM_WIDTH, i * ROOM_HEIGHT, 2);
                            shouldFinal = false;
                        } else {
                            rm = newRoom(roomsX * ROOM_WIDTH, i * ROOM_HEIGHT);
                        }
                        rooms[i].push(rm);
                    }
                    roomsX++;
                } else if (width < (roomsX * ROOM_WIDTH + PADDING_LEFT)) {
                    roomsX--;
                    for (var i=0; i<rooms.length; i++) {
                        rooms[i].splice(roomsX,1);
                    }
                }

                if (height >= (roomsY * ROOM_HEIGHT + ROOM_HEIGHT + PADDING_TOP)) {
                    rooms.push([]);
                    for (var i=0; i<roomsX; i++) {
                        var rm = newRoom(i * ROOM_WIDTH, roomsY * ROOM_HEIGHT);
                        rooms[roomsY].push(rm);
                    }
                    roomsY++;
                } else if (height < (roomsY * ROOM_HEIGHT + PADDING_TOP)) {
                    roomsY--;
                    rooms.splice(roomsY,1);
                }

            }
        }
    }

    // shim layer with setTimeout fallback
    // Paul Irish - http://paulirish.com/2011/requestanimationframe-for-smart-animating/
    window.requestAnimFrame = (function () {
        return window.requestAnimationFrame       ||
               window.webkitRequestAnimationFrame ||
               window.mozRequestAnimationFrame    ||
            function (callback) {
                window.setTimeout(callback, 1000 / fps);
            };
    })();

})();