function Player(o) {
    o = o || {};
    this.ctx = o.context || context;
    this.x = o.x || 0;
    this.y = o.y || 0;
    this.vx = o.vx || 0;
    this.vy = o.vy || 0;
    this.width = o.width || 0;
    this.height = o.height || 0;

    this.attackingWidth = 64;
    this.img = o.image || new Image();
    this.imgGear = o.imgGear || []; //0: sword, 1: shield, 2: helm, 3: armor, 4: boots
    this.type = o.type || 0;
    this.health = o.health || 0;

    this._animation = 0;
    this._MAX_FALL_SPEED = 0.5;
    this._spritex = 0;
    this._spritey = 0;
    this._geary = 0;

    this.attacking = false;
    this.blocking = false;
    this.grounded = false;
    this.dead = false;

    //items
    this.helm = false;
    this.chest = false;
    this.boots = false;
    this.sword = false;
    this.shield = false;
};
Player.prototype.update = function(dt){
    dt = dt || 1;

    //timers
    this._animation += dt;

    if (this.attacking !== false && this.attacking <= 0) {
        this.attacking = false;
    } else if (this.attacking) {
        this.attacking -= dt;    
    }
    
    //positions
    if (this.vy >= this._MAX_FALL_SPEED) {
        this.vy = this._MAX_FALL_SPEED;
    } else {
        this.vy += 0.05; //gravity
    }

    this.x += this.vx * dt;
    this.y += this.vy * dt;
};
Player.prototype.render = function(ctx){
    ctx = ctx || this.ctx || context || false;

    if (ctx) {
        ctx.save();
            ctx.translate(this.x, this.y);
            if (this.vx < 0) {
                ctx.scale(-1,1);
            }


            //status checks
            if (Math.abs(this.vx) > 0.1) {
                this._spritex = 60;
            } else {
                if (this._animation < 800) {
                    this._spritex = 0;
                    this._geary = 0;
                } else if (this._animation < 1000) {
                    this._spritex = 15;
                    this._geary = 2;
                } else if (this._animation > 1000) {
                    this._spritex = 0;
                    this._animation = 0;
                }    
            }

            if (!this.grounded) {
                this._spritex = 45;
            }
            if (this.attacking || this.blocking) {
                this._spritex = 30;
            }

            if (this.dead) {
                this._spritex = 75;
            }

            //under gear render
            if (this.sword) {
                ctx.save();
                    if (!this.dead) {
                        if (this.attacking) {
                            ctx.translate(38, 4);
                            ctx.rotate(1.57);
                            ctx.drawImage(this.imgGear[0], 0, 0, 12, 32); 
                        } else if (this.blocking) {
                            ctx.translate(-24, 16);
                            ctx.rotate(-1.7);
                            ctx.drawImage(this.imgGear[0], 0, 0, 12, 32);
                        } else if (!this.grounded) {
                            ctx.translate(-22, 26);
                            ctx.rotate(-2.2);
                            ctx.drawImage(this.imgGear[0], 0, 0, 12, 32);
                        } else {
                            ctx.translate(-34, 16 + this._geary);
                            ctx.rotate(-1.7);
                            ctx.drawImage(this.imgGear[0], 0, 0, 12, 32);
                        }
                    }
                ctx.restore();
            }

            //toon render
            ctx.drawImage(this.img, this._spritex, this._spritey, 15, 18, -9 /*this.width/2-1*/, -18 /*this.height/2-2*/, 30, 36);

            //over gear
            if (this.boots) {
                ctx.save();
                    if (!this.dead) {
                        if (this.attacking || this.blocking) {
                            ctx.translate(-7, 12);
                            ctx.drawImage(this.imgGear[4], 0, 0, 14, 6); 
                        } else if (!this.grounded) {
                            ctx.translate(-4, 8);
                            ctx.drawImage(this.imgGear[4], 0, 0, 14, 6);
                        } else {
                            ctx.translate(-6, 12);
                            ctx.drawImage(this.imgGear[4], 0, 0, 14, 6);
                        }
                    }
                ctx.restore();
            }

            if (this.armor) {
                ctx.save();
                    if (!this.dead) {
                        if (this.attacking || this.blocking) {
                            ctx.translate(-8, -2);
                            ctx.drawImage(this.imgGear[3], 0, 0, 16, 16); 
                        } else if (!this.grounded) {
                            ctx.translate(-6, -6);
                            ctx.drawImage(this.imgGear[3], 0, 0, 16, 16);
                        } else {
                            ctx.translate(-10, -4 + this._geary);
                            ctx.drawImage(this.imgGear[3], 0, 0, 16, 16);
                        }
                    }
                ctx.restore();
            }

            if (this.helm) {
                ctx.save();
                    if (!this.dead) {
                        if (Math.abs(this.vx) > 0.1) {
                            ctx.translate(-2, -16);
                            ctx.drawImage(this.imgGear[2], 0, 0, 16, 16); 
                        } else if (this.attacking || this.blocking) {
                            ctx.translate(-6, -14);
                            ctx.drawImage(this.imgGear[2], 0, 0, 16, 16); 
                        } else if (!this.grounded) {
                            ctx.translate(-2, -18);
                            ctx.drawImage(this.imgGear[2], 0, 0, 16, 16);
                        } else {
                            ctx.translate(-8, -18 + this._geary);
                            ctx.drawImage(this.imgGear[2], 0, 0, 16, 16);
                        }
                    }
                ctx.restore();
            }

            if (this.shield) {
                ctx.save();
                    if (!this.dead) {
                        if (this.blocking) {
                            ctx.translate(2, -4);
                            ctx.drawImage(this.imgGear[1], 0, 0, 16, 16); 
                        } else if (this.attacking) {
                            ctx.translate(-4, 0);
                            ctx.drawImage(this.imgGear[1], 0, 0, 16, 16); 
                        } else if (!this.grounded) {
                            ctx.translate(4, -2);
                            ctx.drawImage(this.imgGear[1], 0, 0, 16, 16);
                        } else {
                            ctx.translate(-2, 0 + this._geary);
                            ctx.drawImage(this.imgGear[1], 0, 0, 16, 16);
                        }
                    }
                ctx.restore();
            }
        ctx.restore();
    } else {
        console.log('No context specified in the update for:');
        console.log(this);
    }
};

Player.prototype.attack = function(){
    if (this.sword) {
        this.attacking = 500;
    }
};

Player.prototype.block = function(){
    if (this.shield) {
        this.blocking = true;
    }
};
Player.prototype.unblock = function(){
    if (this.shield) {
        this.blocking = false;
    }
};

Player.prototype.jump = function() {
    this.grounded = false;

    if (this.boots) {
        this.vy = -0.8;
    } else {
        this.vy = -0.5;
    }
};