function Room(o) {
    o = o || {};
    this.ctx = o.context || context;
    this.x = o.x || 0;
    this.y = o.y || 0;
    this.width = o.width || 224;
    this.height = o.height || 128;

    this.type = o.type || 0;
    this.bgType = o.bgType || 0; // 1 - color; 2 - image
    this.bg = o.bg || false;
    this.images = o.images;
    
    this.blocks = [];
    if (this.images.length) {
        for (var r=0; r<o.blocks.length; r++) {
            for (var c=0; c<o.blocks[r].length; c++) {

                switch (o.blocks[r][c]) {
                    case 1: //normal block
                        this.blocks.push(new Item({
                            context: this.ctx,
                            type: 0,
                            img: this.images[0],
                            x: c * 16 + 8, 
                            y: r * 16 + 8
                        }));
                    break;
                    case 2: //door
                        this.blocks.push(new Item({
                            context: this.ctx,
                            type: 1,
                            img: this.images[1],
                            x: c * 16 + 8, 
                            y: r * 16 + 8,
                            height: 32
                        }));
                    break;
                    case 3: //key
                        this.blocks.push(new Item({
                            context: this.ctx,
                            type: 2,
                            img: this.images[2],
                            x: c * 16 + 8, 
                            y: r * 16 + 8
                        }));
                    break;
                    case 4: //chest
                        this.blocks.push(new Item({
                            context: this.ctx,
                            type: 3,
                            img: this.images[3],
                            x: c * 16 + 8, 
                            y: r * 16 + 8
                        }));
                    break;
                    case 5: //sword
                        this.blocks.push(new Item({
                            context: this.ctx,
                            type: 4,
                            img: this.images[4],
                            x: c * 16 + 8, 
                            y: r * 16 + 8,
                            width: 12,
                            height: 32
                        }));
                    break;
                    case 6: //shield
                        this.blocks.push(new Item({
                            context: this.ctx,
                            type: 5,
                            img: this.images[5],
                            x: c * 16 + 8, 
                            y: r * 16 + 8
                        }));
                    break;
                    case 7: //boots
                        this.blocks.push(new Item({
                            context: this.ctx,
                            type: 6,
                            img: this.images[6],
                            x: c * 16 + 8, 
                            y: r * 16 + 8
                        }));
                    break;
                    case 8: //helm
                        this.blocks.push(new Item({
                            context: this.ctx,
                            type: 7,
                            img: this.images[7],
                            x: c * 16 + 8, 
                            y: r * 16 + 8
                        }));
                    break;
                    case 9: //armor
                        this.blocks.push(new Item({
                            context: this.ctx,
                            type: 8,
                            img: this.images[8],
                            x: c * 16 + 8, 
                            y: r * 16 + 8
                        }));
                    break;
                    case 10: //spikes
                        this.blocks.push(new Item({
                            context: this.ctx,
                            type: 9,
                            img: this.images[9],
                            x: c * 16 + 8, 
                            y: r * 16 + 8,
                            width: 18,
                            height: 10
                        }));
                    break;
                    case 11:
                        this.blocks.push(new Item({
                            context: this.ctx,
                            type: 10,
                            img: this.images[10],
                            x: c * 16 + 8, 
                            y: r * 16 + 8,
                            width: 32,
                            height: 52
                        }));
                    break;
                    case 12:
                        this.blocks.push(new Monster({
                            context: this.ctx,
                            type: 12,
                            img: this.images[12],
                            x: c * 16 + 8,
                            y: r * 16 + 8,
                            shootTimer: 1000,
                            bulletSpeed: 2
                        }));
                    break;
                    case 13: 
                        this.blocks.push(new Monster({
                            context: this.ctx,
                            type: 12,
                            img: this.images[12],
                            x: c * 16 + 8,
                            y: r * 16 + 8
                        }));
                    break;
                }
            }
        }
    }


    //local vars
    this.blockWidth = 16;
    this.blockHeight = 16;
};
Room.prototype.update = function(delta){
    delta = delta || 1;
};
Room.prototype.render = function(ctx){
    ctx = ctx || this.ctx || context || false;

    if (ctx) {
        ctx.save();
            ctx.translate(this.x, this.y);
            if (this.bg) {
                if (this.bgType === 2) {
                    ctx.drawImage(this.bg, 0, 0, this.width, this.height);
                } else {
                    ctx.fillStyle = this.bg;
                    ctx.fillRect(0, 0, this.width, this.height);
                }
            }

            for (var i=0; i<this.blocks.length; i++) {
                this.blocks[i].render();
            }

        ctx.restore();
    } else {
        console.log('No context specified in the update for:');
        console.log(this);
    }
};